/* BUTTON.C
 * The routine that implements the 'Virtual Button Press' message.
 *
 * Copyright (C) 1991 by Andrew Arensburger. Permission is granted to
 * use, copy and/or distribute this file freely for non-commercial purposes,
 * as long as this notice remains intact. Any commercial use is prohibited
 * without express permission from the author.
 *
 * If you make any changes, fix bugs, etc., please send them to me that I
 * might coordinate fixes and improvements.
 */

#include <stdio.h>
#include "eps.h"

/* PUSH_BUTTON
 * Push button number 'but'.
 * Returns 0 on no error, or a negative error code.
 */
int push_button(chan,but)
int chan;			/* MIDI channel */
int but;			/* Which button? */
{
	int i;

	/* Send instruction to press button */
	send_head(chan);	/* Send SysEx head */
	mpu_sbyte(BUTTON);	/* 'Virtual Button Press' message */
	send12(but);		/* Button number */
	send_tail();		/* Send SysEx tail */

	if (timeout(SHORT_TIMEOUT))
		return(-1);	/* Short timeout occurred */

	/* Receive response to see if everything went well */
	i = recv_response(NULL);

	wait(BREATHER);
	if (i == ACK)
		return(0);		/* All went well */
	else
		return(-1);		/* There was some error */
}
